/*
 * Tp sur le test des objets: tests pour les méthodes de la classe CompteLivreA.
 * Version 2021-2022
 */
package compteLivretA;

import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 * On teste uniquement le constructeur et les méthodes demandées.
 * @author aponte
 */
public class CompteLivretATest {
    
    public CompteLivretATest() {
    }
    
    @BeforeClass
    public static void setUpClass() {
    }
    
    @Before
    public void setUp() {
    }

    
    /* Test du constructeur compte pou état ouvert, il est bien ouvert. */
    @Test
    public void testConstructeurEtatOuvertOK() {
        System.out.println("constructeurEtatOuvertOK");
        CompteLivretA instance = new CompteLivretA("martin", 10, 20);
        assertTrue(instance.estOuvert());
    }
    /* Test du constructeur pour état ouvert, avec solde au plafond. */
    @Test
    public void testConstructeurEtatOuvertSoldeAuPlafond() {
        System.out.println("constructeurEtatOuvertSoldeAuPlafond");
        CompteLivretA instance = new CompteLivretA("martin", 20, 20);
        assertEquals(instance.getSolde(),instance.getPlafond(),0);
    }
     /* Test du constructeur pour état ouvert, avec solde 0. */
    @Test
    public void testConstructeurEtatOuvertSoldeZero() {
        System.out.println("constructeurEtatOuvertSoldeZero");
        CompteLivretA instance = new CompteLivretA("martin", 0, 20);
        assertEquals(instance.getSolde(),0,0);
    }
    
     /* Test du constructeur pour état ouvert, avec solde positif et plus petit que le plafond. */
    @Test
    public void testConstructeurEtatOuvertSoldePos() {
        System.out.println("constructeurEtatOuvertSoldeZero");
        CompteLivretA instance = new CompteLivretA("martin", 10, 20);
        assertTrue(instance.getSolde()<instance.getPlafond());
    }
    /* Test du constructeur pour état ouvert, échec avec plafond à zéro. */
    @Test(expected=IllegalArgumentException.class)
    public void testConstructeurEtatOuvertEchecPlafondZero() {
        System.out.println("testConstructeurEtatOuvertEchecPlafondZero");
        new CompteLivretA("martin", 0, 0);
    }
     /* Test du constructeur pour état ouvert, échec avec plafond négatif. */
    @Test(expected=IllegalArgumentException.class)
    public void testConstructeurEtatOuvertEchecPlafondNeg() {
        System.out.println("testConstructeurEtatOuvertEchecPlafondNeg");
        new CompteLivretA("martin", 0, -1);
    }
       /* Test du constructeur pour état ouvert, solde négatif. */
    @Test(expected=IllegalArgumentException.class)
    public void testConstructeurEtatOuvertEchecSoldeNeg() {
        System.out.println("testConstructeurEtatOuvertEchecSoldeNeg");
        CompteLivretA instance = new CompteLivretA("martin", -1, 10);
    }
   /* Test du constructeur pour état ouvert, solde dépasse le plafond */
    @Test(expected=IllegalStateException.class)
    public void testConstructeurEtatOuvertEchecSoldeDepassement() {
        System.out.println("testConstructeurEtatOuvertEchecSoldeDepassement");
        new CompteLivretA("martin", 20, 10);
    }
    
     /**
     * Tests constructeur compte en état fermé et solde zéro.
     */
    
    @Test
    public void testConstructeurEtatFermeOK() {
        System.out.println("constructeurEtatFermeOK");
        CompteLivretA instance = new CompteLivretA("martin", 20);
        assertTrue(!instance.estOuvert());
    }
    
    @Test(expected=IllegalArgumentException.class)
    public void testConstructeurEtatFermeEchecPlafondZero() {
        System.out.println("testConstructeurEtatFermeEchecPlafondZero");
        new CompteLivretA("martin", 0);
    }
    
    @Test(expected=IllegalArgumentException.class)
    public void testConstructeurEtatFermeEchecPlafondNeg() {
        System.out.println("testConstructeurEtatFermeEchecPlafondNeg");
        new CompteLivretA("martin", -1);
    }


    /**
     * Test of depot method, contrat sur le solde ok
     */
    @Test
    public void testDepotOKEtatSolde() {
        System.out.println("depot");
        CompteLivretA c = new CompteLivretA("martin", 10, 20);
        c.depot(5);
        assertEquals(15, c.getSolde(),0);
    }
    /**
     * Test of depot method, contrat invariant sur le solde ok
     */
    @Test
    public void testDepotOKInvariantSolde() {
        System.out.println("depot");
        CompteLivretA c = new CompteLivretA("martin", 10, 20);
        c.depot(5);
        assertTrue(c.getSolde()<=c.getPlafond());
    }
     /**
     * Test of depot method, le solde arrive au  plafond
     */
    @Test
    public void testDepotAuPlafond() {
        System.out.println("depot");
        CompteLivretA c = new CompteLivretA("martin", 10, 20);
        c.depot(10);
        assertEquals(c.getSolde(),c.getPlafond(),0);
    }
    /**
     * Test of depot method, le montant du dépot est négatif
     */
    @Test(expected=IllegalArgumentException.class)
    public void testDepotFailMontantNegatif() {
        System.out.println("depot");
        CompteLivretA c = new CompteLivretA("martin", 10, 20);
        c.depot(-5);
        assertTrue(c.getSolde()<=c.getPlafond());
    }
    
    /**
     * Test of depot method, le montant fait dépasser le plafond
     */
    @Test(expected=OperationInterdite.class)
    public void testDepotFailDepassementPlafond() {
        System.out.println("depot");
        CompteLivretA c = new CompteLivretA("martin", 10, 20);
        c.depot(15);
    }
    
    /**
     * Test of depot method, le compte est fermé
     */
    @Test(expected=OperationInterdite.class)
    public void testDepotFailCpmteFerme() {
        System.out.println("depot");
        CompteLivretA c = new CompteLivretA("martin", 20);
        c.depot(15);
    }

    /**
     * Test of retrait method, test contrat solde.
     */
    @Test
    public void testRetraitOK() {
        System.out.println("retrait");
        CompteLivretA c = new CompteLivretA("martin", 20, 20);
        c.retrait(5);
        assertEquals(15, c.getSolde(),0);
    }
    /**
     * Test of retrait method, test invariant solde.
     */
    @Test
    public void testRetraitOKInv() {
        System.out.println("retrait");
        CompteLivretA c = new CompteLivretA("martin", 20, 20);
        c.retrait(5);
        assertTrue(c.getSolde()<=c.getPlafond());
    }
    
    /**
     * Test of retrait method, montant négatif.
     */
    @Test(expected=IllegalArgumentException.class)
    public void testRetraitMontantNegatif() {
        System.out.println("retrait");
        CompteLivretA c = new CompteLivretA("martin", 20, 20);
        c.retrait(-5);
    }
     /**
     * Test of retrait method, le montant fait passer en débiteur
     */
    @Test(expected=OperationInterdite.class)
    public void testRetraitFailDevientDebiteur() {
        System.out.println("depot");
        CompteLivretA c = new CompteLivretA("martin", 10, 20);
        c.retrait(15);
    }
    
    /**
     * Test of retrait method, le compte est fermé
     */
    @Test(expected=OperationInterdite.class)
    public void testRetraitFailCpmteFerme() {
        System.out.println("depot");
        CompteLivretA c = new CompteLivretA("martin", 20);
        c.retrait(0);
    }

   
    /**
     * Test of ouvrir method, le compte a été ouvert
     */
    @Test
    public void testOuvrir() {
        System.out.println("ouvrir");
        CompteLivretA c = new CompteLivretA("martin", 20);
        c.ouvrir();
        assertTrue(c.estOuvert());
    }
    /**
     * Test of ouvrir method, le compte a été ouvert
     */
    @Test
    public void testOuvrirDejaOuvert() {
        System.out.println("ouvrir");
        CompteLivretA c = new CompteLivretA("martin", 10, 20);
        c.ouvrir();
        assertTrue(c.estOuvert());
    }
    /**
     * Test of fermer method, le compte peut être fermé.
     */
    @Test
    public void testFermerOK() {
        System.out.println("fermer");
        CompteLivretA c = new CompteLivretA("martin", 0, 20);
        c.fermer();
        assertFalse(c.estOuvert());
    }
    
     /**
     * Test of fermer method, le compte est déjà fermé.
     */
    @Test
    public void testFermerOKDejaFerme() {
        System.out.println("fermer");
        CompteLivretA c = new CompteLivretA("martin", 20);
        c.fermer();
        assertFalse(c.estOuvert());
    }
    
    /**
     * Test of fermer method, le compte ne peut pas être fermé car solde créditeur.
     */
    @Test(expected=OperationInterdite.class)
    public void testFermerFailNonZero() {
        System.out.println("fermer");
        CompteLivretA c = new CompteLivretA("martin", 10, 20);
        c.fermer();
    }
    
}
